import { z } from 'zod';

type EventMap = Record<string, z.ZodTypeAny>;
interface EventEmitterOptions<IncomingEvents extends EventMap = EventMap, OutgoingEvents extends EventMap = EventMap> {
    incomingEvents?: IncomingEvents;
    outgoingEvents?: OutgoingEvents;
}
interface SendActionOptions<IncomingEvents extends EventMap, R extends keyof IncomingEvents> {
    timeoutMs?: number;
    intervalMs?: number;
    condition?: (data: z.infer<IncomingEvents[R]>) => boolean;
}
type SendActionArgs<IncomingEvents extends EventMap, OutgoingEvents extends EventMap, K extends keyof OutgoingEvents, R extends keyof IncomingEvents> = {
    event: K;
    data: z.infer<OutgoingEvents[K]>;
    responseEvent: R;
    options?: SendActionOptions<IncomingEvents, R>;
};
interface OnActionOptions<OutgoingEvents extends EventMap, R extends keyof OutgoingEvents> {
    timeoutMs?: number;
    condition?: (data: z.infer<OutgoingEvents[R]>) => boolean;
}
type OnActionArgs<IncomingEvents extends EventMap, OutgoingEvents extends EventMap, K extends keyof IncomingEvents, R extends keyof OutgoingEvents> = {
    event: K;
    responseEvent: R;
    callback: (data: z.infer<IncomingEvents[K]>) => z.infer<OutgoingEvents[R]>;
    options?: OnActionOptions<OutgoingEvents, R>;
} | {
    event: K;
    options?: OnActionOptions<OutgoingEvents, R>;
};
declare class EventEmitter<IncomingEvents extends EventMap, OutgoingEvents extends EventMap> {
    protected otherWindow: Window;
    protected targetOrigin: string;
    protected incomingEvents: IncomingEvents;
    protected outgoingEvents: OutgoingEvents;
    private listeners;
    constructor(otherWindow: Window, targetOrigin: string, incomingEvents: IncomingEvents, outgoingEvents: OutgoingEvents);
    send<K extends keyof OutgoingEvents>(event: K, data: z.infer<OutgoingEvents[K]>): void;
    on<K extends keyof IncomingEvents>(event: K, callback: (data: z.infer<IncomingEvents[K]>) => void): string;
    sendAction<K extends keyof OutgoingEvents, R extends keyof IncomingEvents>({ event, data, responseEvent, options, }: SendActionArgs<IncomingEvents, OutgoingEvents, K, R>): Promise<z.infer<IncomingEvents[R]>>;
    onAction<K extends keyof IncomingEvents, R extends keyof OutgoingEvents>({ event, options, ...params }: OnActionArgs<IncomingEvents, OutgoingEvents, K, R>): Promise<z.infer<IncomingEvents[K]>>;
    off(id: string): void;
    protected isTargetOrigin(otherOrigin: string): boolean;
}

declare class HandshakeChild<IncomingEvents extends EventMap, OutgoingEvents extends EventMap> extends EventEmitter<IncomingEvents, OutgoingEvents> {
    protected otherWindow: Window;
    protected targetOrigin: string;
    handshakeOptions: Required<HandshakeOptions>;
    isConnected: boolean;
    constructor(otherWindow: Window, targetOrigin: string, options?: EventEmitterWithHandshakeOptions<IncomingEvents, OutgoingEvents>);
    handshakeWithParent(): Promise<void>;
    private _onAction;
}

declare class HandshakeParent<IncomingEvents extends EventMap, OutgoingEvents extends EventMap> extends EventEmitter<IncomingEvents, OutgoingEvents> {
    protected otherWindow: Window;
    protected targetOrigin: string;
    handshakeOptions: Required<HandshakeOptions>;
    isConnected: boolean;
    constructor(otherWindow: Window, targetOrigin: string, options?: EventEmitterWithHandshakeOptions<IncomingEvents, OutgoingEvents>);
    handshakeWithChild(): Promise<void>;
    private _send;
    private _sendAction;
}

interface HandshakeOptions {
    timeoutMs?: number;
    intervalMs?: number;
}
type EventEmitterWithHandshakeOptions<IncomingEvents extends EventMap = EventMap, OutgoingEvents extends EventMap = EventMap> = EventEmitterOptions<IncomingEvents, OutgoingEvents> & {
    handshakeOptions?: HandshakeOptions;
};
declare const DEFAULT_HANDSHAKE_OPTIONS: Required<HandshakeOptions>;
declare const HANDSHAKE_EVENTS: {
    fromChild: {
        readonly handshakeResponse: z.ZodObject<{
            requestVerificationId: z.ZodString;
        }, "strip", z.ZodTypeAny, {
            requestVerificationId: string;
        }, {
            requestVerificationId: string;
        }>;
    };
    fromParent: {
        readonly handshakeRequest: z.ZodObject<{
            requestVerificationId: z.ZodString;
        }, "strip", z.ZodTypeAny, {
            requestVerificationId: string;
        }, {
            requestVerificationId: string;
        }>;
        readonly handshakeComplete: z.ZodObject<{
            requestVerificationId: z.ZodString;
        }, "strip", z.ZodTypeAny, {
            requestVerificationId: string;
        }, {
            requestVerificationId: string;
        }>;
    };
};
type HandshakeEvents = typeof HANDSHAKE_EVENTS.fromParent & typeof HANDSHAKE_EVENTS.fromChild;
type HandshakeParentEvents = typeof HANDSHAKE_EVENTS.fromParent;
type HandshakeChildEvents = typeof HANDSHAKE_EVENTS.fromChild;

declare class ChildWindow<IncomingEvents extends EventMap, OutgoingEvents extends EventMap> extends HandshakeChild<IncomingEvents, OutgoingEvents> {
    private constructor();
    static init<IncomingEvents extends EventMap, OutgoingEvents extends EventMap>(parentWindow: Window, targetOrigin: string, options?: EventEmitterWithHandshakeOptions<IncomingEvents, OutgoingEvents>): Promise<ChildWindow<IncomingEvents, OutgoingEvents>>;
}

declare class IFrameWindow<IncomingEvents extends EventMap, OutgoingEvents extends EventMap> extends HandshakeParent<IncomingEvents, OutgoingEvents> {
    iframe: HTMLIFrameElement;
    targetOrigin: string;
    private constructor();
    static init<IncomingEvents extends EventMap, OutgoingEvents extends EventMap>(url: string, options?: EventEmitterWithHandshakeOptions<IncomingEvents, OutgoingEvents>): Promise<IFrameWindow<IncomingEvents, OutgoingEvents>>;
}

interface PopupWindowOptions {
    width: number;
    height: number;
}
declare class PopupWindow<IncomingEvents extends EventMap, OutgoingEvents extends EventMap> extends HandshakeParent<IncomingEvents, OutgoingEvents> {
    window: Window;
    targetOrigin: string;
    private constructor();
    static init<IncomingEvents extends EventMap, OutgoingEvents extends EventMap>(url: string, options: PopupWindowOptions & EventEmitterWithHandshakeOptions<IncomingEvents, OutgoingEvents>): Promise<PopupWindow<IncomingEvents, OutgoingEvents>>;
}

export { ChildWindow, DEFAULT_HANDSHAKE_OPTIONS, EventEmitter, EventEmitterOptions, EventEmitterWithHandshakeOptions, EventMap, HANDSHAKE_EVENTS, HandshakeChild, HandshakeChildEvents, HandshakeEvents, HandshakeOptions, HandshakeParent, HandshakeParentEvents, IFrameWindow, OnActionArgs, OnActionOptions, PopupWindow, PopupWindowOptions, SendActionArgs, SendActionOptions };
